
#=======Function===============================================
	pr.OHLC<-function(o,  h,  l,  cl){
		# combine quotes matrix. Quotes pre-reversing
		price <- cbind(Open = rev(o), High = rev(h), Low = rev(l), Close = rev(cl));
		Med <-(price[ ,2] + price[ ,3])/2;
		CO <-price[ ,4] - price[ ,1];
		# add the matrix Med and CO
		price<- cbind(price, Med, CO);
	}
#--2----------------------
	In <- function(p = 16){
	  require(TTR)
	  require(dplyr)
	  require(magrittr)
	  adx <- ADX(price, n = p) %>% as.data.frame %>% 
		mutate(.,oscDX = DIp - DIn) %>% 
		transmute(.,DX, ADX, oscDX) %>% 
		as.matrix()
	  ar <- aroon(price[ ,c('High', 'Low')], n = p) %>% 
		extract(,3)
	  atr <- ATR(price, n = p, maType = "EMA") %>%
		extract(,1:2)
	  cci <- CCI(price[ ,2:4], n = p)
	  chv <- chaikinVolatility(price[ ,2:4], n = p)
	  cmo <- CMO(price[ ,'Med'], n = p)
	  macd <- MACD(price[ ,'Med'], 12, 26, 9) %>% 
		as.data.frame() %>% 
		mutate(., vsig = signal %>% 
				 diff %>% c(NA,.) %>% multiply_by(10)) %>% 
		transmute(., sign = signal, vsig) %>% 
		as.matrix()
	  rsi <- RSI(price[ ,'Med'], n = p)
	  stoh <- stoch(price[ ,2:4], nFastK = p, 
					nFastD =3, nSlowD = 3, 
					maType = "EMA") %>%
					as.data.frame() %>% 
					mutate(., oscK = fastK - fastD) %>%
					transmute(.,slowD, oscK) %>% 
					as.matrix()
	  smi <- SMI(price[ ,2:4],n = p, nFast = 2, 
				 nSlow = 25, nSig = 9)
	  kst <- KST(price[ ,4])%>% as.data.frame() %>% 
					mutate(., oscKST = kst - signal) %>%
					select(.,oscKST) %>% as.matrix()
	  In <- cbind(adx, ar, atr, cci, chv, cmo, macd, 
				  rsi, stoh, smi, kst)
	  return(In)
	}
	#----3-------------------------------------
	ZZ <- function(pr = price, ch1, mode="m", sync_ch, ch2) {
	  require(TTR)
	  require(magrittr)
	  if (ch1 > 1) ch1 <- ch1/(10 ^ (Dig - 1))
	  if (ch2 > 1) ch2 <- ch2/(10 ^ (Dig - 1))
	  if (mode == "m") {pr <- pr[ ,'Med']}
	  if (mode == "hl") {pr <- pr[ ,c("High", "Low")]}
	  if (mode == "cl") {pr <- pr[ ,c("Close")]}
	  zz <- ZigZag(pr, change = ch1, percent = F, 
				   retrace = F, lastExtreme = T)
	  zz2 <- ZigZag(pr, change = ch2, percent = F, 
	                retrace = F, lastExtreme = T)
	  n <- 1:length(zz)
	  dz1 <- zz %>% diff %>% c(., NA)
	  dz2 <- zz2 %>% diff %>% c(., NA)
	  sig_o<-0
	  for(i in n)
	  {
	    sig_o[i]=NA;
	    if(!is.na(dz1[i]))
	    {
	      if(dz1[i] > 0){ sig_o[i] = 1; }
	      if(dz1[i] < 0){ sig_o[i] = -1; }
	      #sync
	      if(sync_ch)
	      {
	        if(!is.na(dz2[i]))
	        {
	          if(dz2[i] > 0 && dz1[i] > 0){ sig_o[i] = 1; }
	          if(dz2[i] < 0 && dz1[i] < 0){ sig_o[i] = -1; }
	        }
	      }
	    }
	  }

	 sig <- sign(dz1)

	  for (i in n) { if (is.na(zz[i])) zz[i] = zz[i - 1]}
	  return(cbind(zz, sig_o))
	}
	#-----4---------------------------------
	form.data <- function(n = 16, z = 37, len = 500){
	  require(magrittr)
	  x <- In(p = n)
	  #ln <- len-1
	  out <- ZZ(ch1 = ch1, mode = "m",sync_ch=sync_ch, ch2 = ch2)
	  simpleLabel <- Hmisc::Lag( ifelse( price[, "CO" ] > 0, 1, -1 ), -1 )
	  out <- cbind(NA, simpleLabel) #simple label override, next bar Close-Open
	  data <- cbind(x, y = out[ ,2]) %>%
	    #as.data.frame %>% head(., (nrow(x)-ln))%>%
		  as.data.frame %>%
		    na.omit
	  data$y <- as.factor(data$y)
	  #data$y <- factor( data$y, c("-1","1","2") )
	  return(data)
	}
	#---5---Cleaning------------------------------------------
	cleaning <- function(n = 16, z = 25, cut = 0.9){
	  data <- form.data(n, z)
	  descCor <- cor(data[ ,-ncol(data)])
	  #summary(descCor[upper.tri(descCor)])
	  highCor <- caret::findCorrelation(descCor, cutoff = cut)
	  data <- data[ ,-highCor]
	  #descCor <- cor(data.f[ ,-ncol(data.f)])
	  #summary(descCor[upper.tri(descCor)])
	  return(data)
	}
	#----6---Choosing the Best----------------------------
	prepareBest <- function(n, z, cut, norm, meth)
	{
	  require(randomUniformForest) 
	  data.f <<- cleaning(n = n, z = z, cut = cut)
	  idx <- rminer::holdout(y = data.f$y)
	  x.train <- data.f[idx$tr, -ncol(data.f)]
	  x.test <-  data.f[idx$ts, -ncol(data.f)]
	  y.train <- data.f[idx$tr, ncol(data.f)]
	  y.test <- data.f[idx$ts, ncol(data.f)]
	  if(norm){
		prep <- caret::preProcess(x = x.train, method = meth)
		#c("center", "scale","spatialSign"), "range"
		x.train <- predict(prep, x.train)
		x.test <- predict(prep, x.test)
	  }
	  #---------
	  ruf <- randomUniformForest( X = x.train, 
								  Y = y.train,
								  xtest = x.test, 
								  ytest = y.test,
								  mtry = 1, ntree = 300,
								  threads = 2, 
								  nodesize = 1
	  )
	  imp.ruf <- importance(ruf, Xtest = x.test)
	  best <- imp.ruf$localVariableImportance$classVariableImportance %>% 
		head(., 10) %>% rownames()
	  return(best)
	}
	#-------7---------------
	prepareTrain <- function(x , y, 
							 rati, mod = "stratified", 
							 balance = F, 
							 norm, meth)
	{
	  t <- rminer::holdout(y = y, ratio = rati,
						   mode = mod)
	  train <- cbind(x[t$tr, ], y = y[t$tr])
	  if(balance){
		train <- caret::upSample(x = train[ ,best], 
								 y = train$y, 
								 list = F)%>% tbl_df
		train <- cbind(train[ ,best], select(train, y = Class))
	  }
	  test <- cbind(x[t$ts, ], y = y[t$ts])
	  if (norm) {
		prepr <<- caret::preProcess(train[ ,best], method = meth)
		train = predict(prepr, train[ ,best])%>% cbind(., y = train$y)
		test =  predict(prepr, test[ ,best] %>% cbind(., y = test$y))
	  }
	  DT <- list(train = train,
				 test = test)
	  return(DT)
	}
	#---train SAE--------------------------------------------------
	train_SAE <- function(dt,  
	                      h = c(70, 70, 70), LR = 0.9, Mom = 0.5,
	                      act = "tanh", out = "sigm", sae = "linear", 
	                      Ep = 250, Bs = 50, hd.out = 0.0, in.out = 0.0, bst, sig_type = 0){
	  require(deepnet)
	  x <- dt$train[ ,bst]%>% as.matrix
	  if (soft) { y <- dt$train$y %>% classvec2classmat()}
	  if (!soft){ y <- dt$train$y %>% as.integer() %>% subtract(1)}
	  #Обучаем модель
	  if(sig_type == 0){
	    sae <- sae.dnn.train(x = x , y = y , hidden = h,
	                         activationfun = act, output = out, sae_output = sae,
	                         learningrate = LR, learningrate_scale = 1,
	                         momentum = Mom,
	                         numepochs = Ep, batchsize = Bs,
	                         hidden_dropout = hd.out, visible_dropout = in.out)
	  }
	  if(sig_type == 8){
	    sae <- dbn.dnn.train(x = x , y = y , hidden = h,
	                         activationfun = act, output = out, 
	                         learningrate = LR, learningrate_scale = 1,
	                         momentum = Mom,
	                         numepochs = Ep, batchsize = Bs,
	                         hidden_dropout = hd.out, visible_dropout = in.out)
	  }
	  if(sig_type == 16){
	    sae <- nn.train(x = x , y = y , hidden = h,
	                         activationfun = act, output = out, 
	                         learningrate = LR, learningrate_scale = 1,
	                         momentum = Mom,
	                         numepochs = Ep, batchsize = Bs,
	                         hidden_dropout = hd.out, visible_dropout = in.out)
	  }
	  return (sae)
	}
	#----10-------------------------------------------
	prepareTest <- function(n, z, norm, len = 501)
	{
	  x <- In(p = n )
	  CO <- price[ ,"CO"]
	  prex <-cbind(x = x, CO=CO) %>% na.omit
	  
	  x <- prex %>% extract( ,best) %>% 
	    tail(., len)
	  
	  CO <- prex[ ,"CO"] %>% tail(., len)
	  if (norm) {
	    x <- predict(prepr,x)
	  }
	  dt <- cbind(x = x, CO = CO) %>% as.data.frame()
	  return(dt)
	}
	#---11-----
	testAcc <- function(obj, testData){
	  require(deepnet)
	  x.ts <- testData[ ,best] %>% as.matrix()
	  y.ts <- testData$y %>% as.integer() %>% subtract(1)
	  out <- nn.predict(obj, x.ts) 
	  
	  if(soft){
	    out <- max.col(out)-1
	  } else {
	    out %<>% as.vector()
	    #out <- ifelse(out > mean(out), 1, 0);
	    out <- ifelse(out > 0.5, 0, 1)
	  }
	  
	  acc <- length(y.ts[y.ts == out])/length(y.ts) %>% 
	    round(., digits = 4)
	  return(list(Acc = acc, y.ts = y.ts, y = out))
	}
	#---12-----
	testBal <- function(obj, typ = "bin", dt, bst) {
	  require(deepnet)
	  x <- dt$test[ ,bst]
	  CO <- dt$test$CO
	  out <- nn.predict(obj, x)
	  if(soft){
	    pr.out <- max.col(out)-1; #max.col = argmax
	    if(exists("enable_dn_probability")){
	      if(enable_dn_probability){
	        n_pr <- 1:length(pr.out);
	        sig<-0;
	        for(i in n_pr)
	        {
	          j <- pr.out[i] + 1;
	          sig[i] <- 0;
	          if(!is.na(pr.out[i]) && !is.na(out[i, j])){
	            if(pr.out[i] == 0) if(out[i, j] > dn_probability) sig[i] <- -1;
	            if(pr.out[i] == 1) if(out[i, j] > dn_probability) sig[i] <- 1;
	          }
	        }
	        sig %<>% as.vector()  
	      } else{ sig <- ifelse(pr.out == 0, -1, ifelse(pr.out == 1, 1, 0) ) }
	    } else { sig <- ifelse(pr.out == 0, -1, ifelse(pr.out == 1, 1, 0) ) }
	    
	    sig %<>% as.vector() 
	  } else {
	    pr.out <- out %>% as.vector()
	    
	    if(exists("dec")){
	      if(dec == 2) sig <- ifelse(pr.out > left, -1, ifelse(pr.out < right, 1, 0))
	      if(dec == 1) sig <- ifelse(pr.out > 0.5, -1, 1)
	    } else sig <- ifelse(pr.out == 1, -1, 1)
	  }
	  sig1 <- Hmisc::Lag(sig) %>% na.omit
	  #
	  if(is.null(CO)){
	    Kmax <- 0
	    K <- 0
	    Acc <- 0
	    maxDD<-0
	    bal<-0
	    K<-0
	    dd<-0
	  } else {
	    
	    #bal <- cumsum(sig1 * tail(CO, length(sig1)))
	    bal <- tryCatch({
	      bal <- cumsum(sig1 * tail(CO, length(sig1)))
	    }, error = function(e){
	      bal<-0
	      return(bal)})
	    #K <- tail(bal, 1)/length(bal) * 10 ^ Dig
	    K <- tryCatch({
	      K <- tail(bal, 1)/length(bal) * 10 ^ Dig
	    },error = function(e){
	      K<-0
	      return(K)})
	    #Kmax <- max(bal)/which.max(bal) * 10 ^ Dig
	    Kmax <- tryCatch({
	      Kmax <- max(bal)/which.max(bal) * 10 ^ Dig
	    },error = function(e){
	      Kmax<-0
	      return(Kmax)})
	    #dd <- fTrading::maxDrawDown(bal)
	    dd <- tryCatch({
	      dd <- fTrading::maxDrawDown(bal)
	    },error = function(e){
	      dd<-0
	      return(dd)})
	  }
	  return(list(sig = sig, bal = bal, K = K, 
	              Kmax = Kmax, dd = dd))
	}
	#---13----------------------------------
	sig2stat <- function(x) {
	  use3pos<-FALSE
	  use3pos<- '0' %in% x
	  if(use3pos){
	    return(ifelse(x==-1, 1, ifelse(x==1, 0, 2)) %>% as.numeric)   
	  } else {
	    return (x %>% as.factor %>% as.numeric)
	  }
	  
	}
	stat2sig <- function(x) {
	  use3pos<-FALSE
	  use3pos<- '2' %in% x
	  if(use3pos){
	    return(ifelse(x==1, -1, ifelse(x==0, 1, 0)))
	  } else {
	    return(ifelse(x==1, -1, 1))
	  }
	  #ifelse(x==1, -1, 1)
	}
	#----14--correct-----------------------------------
	correct <- function(sig){
	  sig <- Hmisc::Lag(sig) %>% na.omit
	  bal <- cumsum(sig * (price[ ,6] %>% tail(.,length(sig))))
	  ma <- pracma::movavg(bal, 16, "t")
	  roc <- TTR::momentum(ma, 3)%>% na.omit
	  dbal <- (bal - ma) %>% tail(., length(roc))
	  dbr <- (roc + dbal) %>% as.matrix()
	  sig.cor <- sign(dbr)
	  sig.cor <- Hmisc::Lag(sig.cor) %>% na.omit
	  S <- sig.cor * tail(sig, length(sig.cor))
	  bal <- cumsum(S * (price[ ,6]%>% tail(.,length(S))))
	  K <- tail(bal, 1)/length(bal) * 10 ^ Dig
	  Kmax <- max(bal)/which.max(bal) * 10 ^ Dig
	  dd <- fTrading::maxDrawDown(bal)
	  corr <<- list(sig.c = sig.cor, sig.res = S, bal = bal, Kmax = Kmax, K = K, dd = dd)
	  return(corr)
	}

	#---15---markovchain----------------------------------
	pred.sig <- function(sig, prev.bar = 10, nahead = 1){
	  require(markovchain)
	# Переведем наблюдаемые корректирующие сигналы в состояния 
	  stat <- sig2stat(sig)
	# Вычислим параметры модели
	if(!exists('MCbsp')){
	  MCbsp <<- markovchainFit(data = stat, 
							method = "bootstrap",
							nboot = 10L,
							name="Bootstrap_MarkovChain")
	}
	# Зададим необходимые константы
	  newData <- tail(stat, prev.bar)
	  pr <- predict(object = MCbsp$estimate, 
					newdata = newData,
					n.ahead = nahead)
	# пристыкуем предсказанный сигнал к входному
	  sig.pr <- c(sig, stat2sig(pr))
	  return(sig.pr = sig.pr)
	}
	#---16---smooth------------------------------------
	smoooth <- function(sig){
	# сгладим предсказанный сигнал
	# определим параметры скрытой модели Мвркова
	  require(mhsmm)
	if(!exists('obj.sm')){
	  obj.sm <<- sig2stat(sig)%>% smooth.discrete()
	 }
	# сгладим сигнал полученной моделью
	  sig.s <- predict(obj.sm, x = sig2stat(sig))%>% 
				extract2(1)%>% stat2sig()
	# вычислим баланс по сглаженному сигналу
	  sig.s1 <- Hmisc::Lag(sig.s) %>% na.omit
	  bal <- cumsum(sig.s1 * (price[ ,6]%>% tail(.,length(sig.s1))))
	  K <- tail(bal, 1)/length(bal) * 10 ^ Dig
	  Kmax <- max(bal)/which.max(bal) * 10 ^ Dig
	  dd <- fTrading::maxDrawDown(bal)
	  return(list(sig = sig.s, bal = bal, Kmax = Kmax, K = K, dd = dd))
	}
	
	euc_dist <- function(m)
	{
	  mtm <- Matrix::tcrossprod(m)
	  sq <- rowSums(m*m)
	  return(sqrt(outer(sq,sq,"+") - 2*mtm))
	} 
	
	#uses only dataCols columns for dist calculation
	uniqueTD <- function(trainData, dataCols)
	{
	  require(wordspace)
	  #select rows with y == -1 and 1
	  #class0Entries <- trainData[trainData$y == -1, !names(trainData) %in% c("y")]
	  #class1Entries <- trainData[trainData$y == 1, !names(trainData) %in% c("y")]
	  class0Entries <- trainData[trainData$y == -1, dataCols] #will be rows in distances
	  class1Entries <- trainData[trainData$y == 1, dataCols] #will be cols
	  #convert to matrix before compute?
	  
	  #calculate euclidean distances from each to each
	  #distances is matrix(data.frame?)
    #distances <- apply(class0Entries, 1, function(class0Row) apply(class1Entries, 1, function(class1Row) dist.matrix( data.matrix( rbind(class0Row, class1Row) ), as.dist=T, method = "euclidean" )[1] ) )
	  #distances <- apply(class0Entries, 1, function(class0Row) apply(class1Entries, 1, function(class1Row) dist( rbind(class0Row, class1Row) )[1] ) )
    distances <- rdist::cdist(class0Entries, class1Entries)
    minDist = as.numeric( quantile(as.vector(distances), 0.01/10) )
    filteredRows <- vector("character", nrow(trainData)) #row names
    curIdx <- 1
    for (i in 1:nrow(distances))
    {
      filtered = F
      curRow <- distances[i,]
      for ( j in 1:length(curRow) )
      {
        if (curRow[j] < minDist)
        {
          filtered = T
          filteredRows[curIdx] <- rownames(class1Entries)[j]
          curIdx <- curIdx + 1
        }
      }
      if ( filtered )
      {
        filteredRows[curIdx] <- rownames(class0Entries)[i]
        curIdx = curIdx + 1
      }
    }
    filteredRows <- unique(filteredRows)
	  #print(rownames(class0Entries))
	  #print(distances[1:10,])
	  #print(filteredRows)
    
    #res <- trainData[ !(row.names(trainData) %in% filteredRows), ]
    res <- trainData
    levels(res$y) <- c(levels(res$y),"2")
    index <- row.names(res) %in% filteredRows
    res$y[index] <- 2 #set all filtered to label 2
    return (res)
	}
	
	uniqueTD2 <- function(trainData, dataCols)
	{
	  diffRatio = 0.00000001 #filter relative diff less than this
	  class0Entries <- trainData[trainData$y == -1, dataCols]
	  class1Entries <- trainData[trainData$y == 1, dataCols]
	  class0Entries <- as.matrix(class0Entries)
	  class1Entries <- as.matrix(class1Entries)
	  
	  curIdx0 <- 0
	  curIdx1 <- 0
	  filteredRows <- vector("character", nrow(trainData)) #row names
	  filteredIdxs0 <- vector()
	  filteredIdxs1 <- vector()
	  for ( i in 1:nrow(class0Entries) ) #for each column
	  {
	    for ( j in 1:nrow(class1Entries) ) #for each row of first
	    {
	      filtered = F
	      cond <- ( abs(class0Entries[i,] - class1Entries[j,]) < (class0Entries[i,] * diffRatio) )
	      for ( k in 1:ncol(class1Entries) ) #for each row of second
        {
	        #filter elems from different classes if difference is less than diffRatio of first
	        if ( cond[k] )
	        {
	          filtered = T
	          filteredIdxs1[curIdx1] <- j #filter second
	          curIdx1 <- curIdx1 + 1
	          break
	        }
	      }
	      if ( filtered )
	      {
	        filteredIdxs0[curIdx0] <- i #filter first
	        curIdx0 = curIdx0 + 1
	      }
	    }
	  }
	  
	  #for (i in 1:length(filteredIdxs0))
	  #{
	   # filteredRows[curFRIdx] <- rownames(class0Entries)[filteredIdxs0[i]]
	   # curFRIdx <- curFRIdx+1
	  #}
	  #convert indexes to row names
	  filteredIdxs0 <-  unique(filteredIdxs0) 
	  filteredIdxs1 <-  unique(filteredIdxs1) 
	  filteredRows <- lapply(filteredIdxs0, function(x) rownames(class0Entries)[x])
	  filteredRows <- c( filteredRows, lapply(filteredIdxs1, function(x) rownames(class1Entries)[x]) )
	  
	  filteredRows <- unique(filteredRows)
	  res <- trainData
	  levels(res$y) <- c(levels(res$y),"2")
	  index <- row.names(res) %in% filteredRows
	  res$y[index] <- 2 #set all filtered to label 2
	  return (res)
	}
	
	#macd signal data
	MACDSignal <- function(param)
	{
	  require(TTR)
	  require(magrittr)
	  # определим переменные
	  x <- pr[param[1]]
	  nFast <- param[2]
	  nSlow <- param[3]
	  nSig <- param[4]
	  macdType <- MaType[param[5]]
	  sigType <- MaType[param[6]]
	  percent <- per[param[7]]
	  len <- 10*100 #param[9] * 100
	  # линейное ограничение для macd
	  if (nSlow <= nFast)
	    return(list(avail=F))
	  # вычисляем macd
	  md <- MACD(
	    x = x,
	    nFast = nFast,
	    nSlow = nSlow,
	    nSig = nSig,
	    percent = TRUE,
	    maType = list(list(macdType),
	                  list(macdType),
	                  list(sigType))
	  )
	  # вычисляем сигналы
	  sig <- signal(md, param[8])
	  return (list(sig = sig, avail = T, len = len))
	}
	# фитнес функция-------------------------
	#MACD fitness
	fitnes <- function(param, test = FALSE)
	{
	  sigRes <- MACDSignal(param)
	  if (sigRes$avail == FALSE)
	    return(-Inf)
	  sig <- sigRes$sig
	  len <- sigRes$len
	  # и сдвигаем вправо на 1 бар
	  sig <- Lag(sig) %>% na.omit()
	  #вычисляем баланс на истории длиной len
	  bal <- cumsum(tail(sig, len) * tail(price[, 'CO'], len))
	  if (test)
	  {
	    bal <<- cumsum(tail(sig, len) * tail(price[, 'CO'], len))
	  }
	  # вычисляем коэффициент качества(округляем до целого числа)
	  K <- ((tail(bal, 1) / length(bal)) * 10 ^ Dig) %>% floor()
	  plot(bal)
	  # возвращаем полученный критерий оптимизации
	  return(unname(K))
	}
	
	#MACD Signal type
	signal <- function(x, type){
	  x <- na.omit(x)
	  dx <- diff(x[ ,1]) %>% na.omit()
	  x <- tail(x, length(dx))
	  switch(type,
	         (x[ ,1] - x[ ,2]) %>% sign(),
	         sign(dx),
	         ifelse(sign(dx) == 1 & sign(x[ ,1]) == 1, 1,
	                ifelse(sign(dx) == -1 & sign(x[ ,1]) == -1,-1, 0))
	  )
	}
	
	MACDPreparePr<-function()
	{
	  pr <<- transmute(as.data.frame(price), Close = Close, Med = Med, 
	                   Typ = (High + Low + Close)/3,
	                   WClose = (High + Low + 2*Close)/4)
	}
	
	MACDOpt <- function(printDbg=F)
	{
	  require(Hmisc)
	  # Типы средних = 4 -------------------------------------------
	  MaType <<- Cs(SMA, EMA, DEMA, ZLEMA)
	  require(dplyr)
	  # Типы цен = 4 -----------------------------------------------
	  MACDPreparePr()
	  # как считать?
	  per <<- c(TRUE, FALSE)
	  # Варианты сигналов = 3 --------------------------
	  
	  # начальная крнфигунация-------------------------- 
	  par <- c(2, 15, 26, 9, 2, 1, 1, 3, 5) #par <- c(2, 12, 26, 9, 2, 1, 1, 3, 5)
	  # пространство поиска------------------------------
	  dom <- matrix(c(1, 4,   # для типов цен
	                  15, 25, #8, 21,  # для периода быстрой МА
	                  20, 54, # для периода медленной МА
	                  3, 13,  # для периода сигнальной МА
	                  1, 4,   # тип МА для быстрой и медленной
	                  1, 4,   # тип МА для сигнальной
	                  1, 2,   # тип percent
	                  1, 3,   # вариант signal
	                  3,10),  # длина истории [300:1000]
	                ncol = 2, byrow = TRUE)
	  # создать кластер из доступных ядер процессора
	  puskCluster<-function(){
	    library(doParallel)
	    library(foreach)
	    cores<-detectCores()
	    cl<-makePSOCKcluster(cores)
	    registerDoParallel(cl)
	    #clusterSetRNGStream(cl)
	    return(cl)
	  }
	  
	  # запускаем кластер
	  cl <- puskCluster()
	  # максимизация фитнес функции
	  # передаем каждому ядру в кластере необходимые переменные и функции
	  clusterExport(cl, list("price", "pr", "MaType", "par", "dom", "signal", 
	                         "fitnes", "Lag", "Dig", "per",
	                         "MACDSignal"), envir = environment() ) #захватить из текущего окуружения, чтобы захватывать локальные
	  
	  require(rgenoud)
	  printLvl = 0
	  if (printDbg)
	    printLvl = 2
	  pr.max <- genoud(fitnes, nvars = 9, max = TRUE, 
	                   pop.size = 500, max.generation = 300,
	                   wait.generation = 10, 
	                   hard.generation.limit = TRUE,
	                   starting.values = par, Domains = dom, 
	                   boundary.enforcement = 1,
	                   data.type.int = TRUE,
	                   solution.tolerance = 0.01,
	                   cluster = cl,
	                   print.level = printLvl)
	  # останавливаем кластер
	  stopCluster(cl)
	  
	  if (printDbg)
	  {
	    K.opt <- fitnes(pr.max$par, test = TRUE)
	    print(K.opt)
	    plot(bal, t="l")
	  }
	  return (pr.max)
	}